/*
    This file is part of the KDE games library
    Copyright 2008 Andreas Pakulat <apaku@gmx.de>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License version 2 as published by the Free Software Foundation.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#include "carddeckinfo.h"
#include "carddeckinfo_p.h"

#include <QDebug>

#include <QFileInfo>
#include <QDir>
#include <QSettings>
#include <QtGlobal>

#include "krandom/krandom.h"

// KConfig entries
#define CONF_LOCKING QString::fromLatin1("Locking")
#define CONF_ALLOW_FIXED_CARDS QString::fromLatin1("AllowFixed")
#define CONF_CARD QString::fromLatin1("Cardname")
#define CONF_DECK QString::fromLatin1("Deckname")

/**
 * Local static information.
 */
class KCardThemeInfoStatic
{
public:
    KCardThemeInfoStatic()
    {
        readBacks();
        readFronts();
    }
    ~KCardThemeInfoStatic()
    {
    }
    
    QStringList getSvgDesktops()
    {
        QString svgDirPath = QString(DATADIR) + "/carddecks/";
    	QDir svgDir(svgDirPath);
    	QStringList svgDirs = svgDir.entryList(QStringList() << "svg*", QDir::Dirs);
    	QStringList res;
    	foreach (QString dir, svgDirs)
    	{
    		res << svgDirPath + dir + "/index.desktop";
    	}
    	return res;
    }
    
    void readFronts()
    {
    	qDebug() << Q_FUNC_INFO;
        // Empty data
        pngFrontInfo.clear();
        svgFrontInfo.clear();

        QStringList svg;
        // Add SVG card sets
        svg = getSvgDesktops();
        const QStringList list = svg;

	qDebug() << list;

        if ( list.isEmpty() ) return;

        for ( QStringList::ConstIterator it = list.begin(); it != list.end(); ++it )
        {
            qDebug() << *it;
            QSettings cfgcg(*it, QSettings::IniFormat);
            cfgcg.beginGroup("KDE Backdeck");
            QString path = ( *it ).left(( *it ).lastIndexOf( '/' ) + 1 );
            Q_ASSERT( path[path.length() - 1] == '/' );
            QString previewPixmapPath = path + cfgcg.value( "Preview", "12c.png" ).toString();
            qDebug() << previewPixmapPath;
            QPixmap pixmap( previewPixmapPath );
            if ( pixmap.isNull() )
            {
            	qDebug() << "null pixmap";
                continue;
            }

            QString idx  = cfgcg.value( "Name", "unnamed" ).toString();
            QString defaultLocalName = idx.isEmpty() ? QObject::tr( "unnamed" ) : idx;
            QString name = cfgcg.value( "Name[" + QLocale::system().name() + "]", defaultLocalName ).toString();
            KCardThemeInfo info;
            info.name         = name;
            info.noi18Name    = idx;
            info.comment      = cfgcg.value( "Comment", QString() ).toString();
            info.preview      = pixmap;
            info.path         = path;
            info.back         = cfgcg.value( "Back", QString() ).toString();
            // if (!info.back.isNull()) kDebug() << "FOUND BACK " << info.back;
            info.isDefault    = cfgcg.value( "Default", false ).toBool();

            qDebug() << info;

            QString svg    = cfgcg.value( "SVG", QString() ).toString();
            if ( !svg.isEmpty() )
            {
                QFileInfo svgInfo( QDir( path ), svg );
                info.svgfile = svgInfo.filePath();
                svgFrontInfo[idx] = info;
            }
            else
            {
                info.svgfile.clear();
                pngFrontInfo[idx] = info;
            }
        }

    }

    QStringList getDeckDesktops()
    {
    	QString decksDirPath = QString(DATADIR) + "/carddecks/decks/";
        QDir decksDir(decksDirPath);
        QStringList res;
        foreach (QString desktop, decksDir.entryList(QStringList() << "*.desktop"))
        {
        	res << decksDirPath + desktop;
        }
    	
    	return res;
    }

    void readBacks()
    {
    	qDebug() << Q_FUNC_INFO;
        // Empty data
        svgBackInfo.clear();
        pngBackInfo.clear();
    
        const QStringList list = getDeckDesktops();

        if ( list.isEmpty() ) return;
    
        for ( QStringList::ConstIterator it = list.begin(); it != list.end(); ++it )
        {
            qDebug() << *it;
            //KConfig cfg( *it, KConfig::SimpleConfig );
            QString path = ( *it ).left(( *it ).lastIndexOf( '/' ) + 1 );
            Q_ASSERT( path[path.length() - 1] == '/' );
            QPixmap pixmap( getBackFileNameFromIndex( *it ) );
            if ( pixmap.isNull() ) continue;

            QSettings cfgcg(*it, QSettings::IniFormat); //( &cfg, "KDE Cards" );
            cfgcg.beginGroup("KDE Cards");

            QString idx  = cfgcg.value( "Name", "unnamed" ).toString();
            QString defaultLocalName = idx.isEmpty() ? QObject::tr( "unnamed" ) : idx;
            QString name = cfgcg.value( "Name[" + QLocale::system().name() + "]", defaultLocalName ).toString();
            KCardThemeInfo info;
            info.name         = name;
            info.noi18Name    = idx;
            info.path         = getBackFileNameFromIndex( *it );
            info.comment      = cfgcg.value( "Comment", QString() ).toString();
            info.preview      = pixmap;
            info.isDefault    = cfgcg.value( "Default", false ).toBool();
            
            qDebug() << info;
    
            QString svg    = cfgcg.value( "SVG", QString() ).toString();
            qDebug() << "back svg = " << svg;
            if ( !svg.isEmpty() )
            {
                QFileInfo svgInfo( QDir( path ), svg );
                info.svgfile = svgInfo.filePath();
                svgBackInfo[idx] = info;
            }
            else
            {
                info.svgfile.clear();
                pngBackInfo[idx] = info;
            }
        }
    
    }

    QString getBackFileNameFromIndex( const QString& desktop )
    {
        qDebug() << Q_FUNC_INFO;
        QString entry = desktop.left( desktop.length() - strlen( ".desktop" ) );
        qDebug() << entry;
        if ( QFileInfo( entry + QString::fromLatin1( ".png" ) ).exists() )
            return entry + QString::fromLatin1( ".png" );

        // rather theoretical
        /*if ( KStandardDirs::exists( entry + QString::fromLatin1( ".xpm" ) ) )
            return entry + QString::fromLatin1( ".xpm" );*/
   
        Q_ASSERT( false );
        return QString();
    }


    /** The card front sides for PNG decks.
     */
    QMap<QString, KCardThemeInfo> pngFrontInfo;

    /** The card front sides for SVG decks.
     */
    QMap<QString, KCardThemeInfo> svgFrontInfo;

    /** The card back sides for PNG decks.
     */
    QMap<QString, KCardThemeInfo> pngBackInfo;

    /** The card back sides for SVG decks.
     */
    QMap<QString, KCardThemeInfo> svgBackInfo;

    /** The default front side name.
     */
    QString defaultFront;

    /** The default back side name.
     */
    QString defaultBack;
};

static struct DeckInfoStatic
{
	DeckInfoStatic() : m_deckinfoStatic(0) {}
	
	KCardThemeInfoStatic* operator->()
	{
		if (m_deckinfoStatic == 0)
			m_deckinfoStatic = new KCardThemeInfoStatic;
		
		return m_deckinfoStatic;
	}
private:
	KCardThemeInfoStatic* m_deckinfoStatic;
} deckinfoStatic;

//static KCardThemeInfoStatic* deckinfoStatic = new KCardThemeInfoStatic;

QDebug operator<<(QDebug debug, const KCardThemeInfo &cn)
{
    return debug << "name: " << cn.name
                 << " noi18Name: " << cn.noi18Name
                 << " comment: " << cn.comment
                 << " path: " << cn.path
                 << " back: " << cn.back
                 << " preview: " << cn.preview
                 << " svgfile: " << cn.svgfile
                 << " isDefault: " << cn.isDefault;
}

namespace CardDeckInfo
{

// Retrieve default card set name
QString defaultFrontName( bool pAllowPNG )
{
    qDebug() << Q_FUNC_INFO;
    QString noDefault;
    // Count filtered cards
    QMap<QString, KCardThemeInfo> temp = deckinfoStatic->svgFrontInfo;
    qDebug() << "temp.size =" << temp.size();
    if ( pAllowPNG )
    {
        temp.unite( deckinfoStatic->pngFrontInfo );
    }
    QMapIterator<QString, KCardThemeInfo> it( temp );
    while ( it.hasNext() )
    {
        it.next();
        KCardThemeInfo v = it.value();
        qDebug() << v;
        // Filter
        if ( v.isDefault ) return v.noi18Name;
        // Collect any deck if no default is stored
        noDefault = v.noi18Name;
    }
    if ( noDefault.isNull() ) qCritical() << "Could not find default card name";
    return noDefault;
}


// Retrieve default deck name
QString defaultBackName( bool pAllowPNG )
{
    qDebug() << Q_FUNC_INFO;
    QString noDefault;
    QMap<QString, KCardThemeInfo> temp = deckinfoStatic->svgBackInfo;
    qDebug() << "temp.size = " << temp.size();
    if ( pAllowPNG )
    {
        temp.unite( deckinfoStatic->pngBackInfo );
    }

    QMapIterator<QString, KCardThemeInfo> it( temp );
    while ( it.hasNext() )
    {
        it.next();
        KCardThemeInfo v = it.value();
        // Filter
        if ( v.isDefault ) return v.noi18Name;
        // Collect any deck if no default is stored
        noDefault = v.noi18Name;
    }
    if ( noDefault.isNull() ) qCritical() << "Could not find default deck name";
    return noDefault;
}


// Retrieve a random card name
QString randomFrontName( bool pAllowPNG )
{
    // Collect matching items
    QStringList list = deckinfoStatic->svgFrontInfo.keys();
    if ( pAllowPNG )
    {
        list += deckinfoStatic->pngFrontInfo.keys();
    }

    // Draw random one
    int d = KRandom::random() % list.count();
    return list.at( d );
}


// Retrieve a random deck name
QString randomBackName( bool pAllowPNG )
{
    // Collect matching items
    QStringList list = deckinfoStatic->svgBackInfo.keys();
    if ( pAllowPNG )
    {
        list += deckinfoStatic->pngBackInfo.keys();
    }

    // Draw random one
    int d = KRandom::random() % list.count();
    return list.at( d );
}


// Retrieve the PNG filename for a back side from its index.desktop filename
QString getBackFileNameFromIndex( const QString &desktop )
{
    QString entry = desktop.left( desktop.length() - strlen( ".desktop" ) );
    if ( QFileInfo( entry + QString::fromLatin1( ".png" ) ).exists() )
        return entry + QString::fromLatin1( ".png" );

    return QString();
}

// Retrieve the SVG file belonging to the given card back deck.
QString backSVGFilePath( const QString& name )
{
    if ( !deckinfoStatic->svgBackInfo.contains( name ) ) return QString();
    const KCardThemeInfo &v = deckinfoStatic->svgBackInfo[name];
    return v.svgfile;
}

// Retrieve the SVG file belonging to the given card fronts.
QString frontSVGFilePath( const QString& name )
{
    if ( !deckinfoStatic->svgFrontInfo.contains( name ) ) return QString();
    const KCardThemeInfo &v = deckinfoStatic->svgFrontInfo[name];
    return v.svgfile;
}


// Retrieve the PNG file belonging to the given card back deck.
QString backFilename( const QString& name )
{
    if ( !deckinfoStatic->pngBackInfo.contains( name ) ) return QString();
    const KCardThemeInfo &v = deckinfoStatic->pngBackInfo[name];
    return v.path;
}


// Retrieve the directory belonging to the given card fronts.
QString frontDir( const QString& name )
{
    if ( !deckinfoStatic->pngFrontInfo.contains( name ) ) return QString();
    const KCardThemeInfo &v = deckinfoStatic->pngFrontInfo[name];
    return v.path;
}


// Check whether a card set is SVG
bool isSVGFront( const QString& name )
{
    return deckinfoStatic->svgFrontInfo.contains( name );
}


// Check whether a card deck is SVG
bool isSVGBack( const QString& name )
{
    return deckinfoStatic->svgBackInfo.contains( name );
}

QStringList frontNames()
{
    return ( deckinfoStatic->svgFrontInfo.keys() + deckinfoStatic->pngFrontInfo.keys() );
}

QStringList backNames()
{
    return ( deckinfoStatic->svgBackInfo.keys() + deckinfoStatic->pngBackInfo.keys() );
}

KCardThemeInfo frontInfo( const QString& name )
{
    if ( deckinfoStatic->svgFrontInfo.contains( name ) )
        return deckinfoStatic->svgFrontInfo.value( name );
    if ( deckinfoStatic->pngFrontInfo.contains( name ) )
        return deckinfoStatic->pngFrontInfo.value( name );
    return KCardThemeInfo();
}

KCardThemeInfo backInfo( const QString& name )
{
    if ( deckinfoStatic->svgBackInfo.contains( name ) )
        return deckinfoStatic->svgBackInfo.value( name );
    if ( deckinfoStatic->pngBackInfo.contains( name ) )
        return deckinfoStatic->pngBackInfo.value( name );
    return KCardThemeInfo();
}

QString frontTheme( const QSettings& group, const QString& defaultTheme )
{
    qDebug() << Q_FUNC_INFO;
    QString theme = group.value( CONF_CARD, defaultTheme ).toString();
    if (!frontNames().contains(theme))
        return defaultTheme;
    return theme;
}

QString backTheme( const QSettings& group, const QString& defaultTheme )
{
    qDebug() << Q_FUNC_INFO;
    QString theme = group.value( CONF_DECK, defaultTheme ).toString();
    if (!backNames().contains(theme))
        return defaultTheme;
    return theme;
}

bool allowFixedSizeDecks( const QSettings& group, bool lockDefault )
{
  return group.value( CONF_ALLOW_FIXED_CARDS, lockDefault ).toBool();
}

bool lockFrontToBackside(const QSettings& group, bool lockDefault)
{
  return group.value( CONF_LOCKING, lockDefault ).toBool();
}

void writeFrontTheme( QSettings& group, const QString& theme )
{
  group.setValue( CONF_CARD, theme );
}

void writeBackTheme( QSettings& group, const QString& theme )
{
  group.setValue( CONF_DECK, theme );
}

void writeAllowFixedSizeDecks( QSettings& group, bool allowFixedSize )
{
  group.setValue( CONF_ALLOW_FIXED_CARDS, allowFixedSize );
}

void writeLockFrontToBackside( QSettings& group, bool lock )
{
  group.setValue( CONF_LOCKING, lock );
}

}

