/*
   patience -- main program
   Copyright (C) 1995  Paul Olav Tvete <paul@troll.no>

   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted,
   provided that the above copyright notice appear in all copies and that
   both that copyright notice and this permission notice appear in
   supporting documentation.

   This file is provided AS IS with no warranties of any kind.  The author
   shall have no liability with respect to the infringement of copyrights,
   trade secrets or any patents by this file or any part thereof.  In no
   event will the author be liable for any lost revenue or profits or
   other special, indirect and consequential damages.


   Heavily modified by Mario Weilguni <mweilguni@sime.com>
*/

#include "pwidget.h"

#include "cardmaps.h"
#include "dealer.h"
#include "demo.h"
#include "gamestatsimpl.h"
#include "render.h"
#include "version.h"
#include "view.h"

#include "krandom/krandom.h"
#include "libkdegames/carddeckinfo.h"
#include "libkdegames/kcarddialog.h"

#include <QAction>
#include <QApplication>
#include <QSettings>
#include <QDebug>
#include <QFileDialog>
#include <QtGlobal>
#include <QIcon>
#include <QInputDialog>
#include <QMenuBar>
#include <QMessageBox>
#include <QStatusBar>
#include <QToolBar>
#include <QTemporaryFile>
#include <QtCore/QList>
#include <QtCore/QPointer>
#include <QtCore/QTimer>
#include <QDesktopWidget>
#include <QStackedWidget>
#include <QtXml/QDomDocument>


static pWidget *current_pwidget = 0;

pWidget::pWidget()
  : QMainWindow(0),
    dill(0),
    m_dealer(0),
    m_bubbles(0)
{
    setObjectName( "pwidget" );

    current_pwidget = this;
    // KCrash::setEmergencySaveFunction(::saveGame);

    // Game
    newGameAction = new QAction(tr("New"), this);
    newGameAction->setShortcut(tr("Ctrl+N"));
    newGameAction->setIcon(QIcon::fromTheme("document-new"));
    connect(newGameAction, SIGNAL(triggered( bool )), SLOT(newGame()));
    
    restartGameAction = new QAction(tr("Restart"), this);
    restartGameAction->setShortcut(tr("F5"));
    connect(restartGameAction, SIGNAL(triggered( bool )), SLOT(restart()));
    
    loadGameAction = new QAction(tr("Load"), this);
    loadGameAction->setShortcut(tr("Ctrl+O"));
    connect(loadGameAction, SIGNAL(triggered( bool )), SLOT(openGame()));
    
    saveGameAction = new QAction(tr("Save"), this);
    saveGameAction->setShortcut(tr("Ctrl+S"));
    saveGameAction->setIcon(QIcon::fromTheme("document-save"));
    connect(saveGameAction, SIGNAL(triggered( bool )), SLOT(saveGame()));
    
    quitGameAction = new QAction(tr("Quit"), this);
    quitGameAction->setShortcut(tr("Ctrl+Q"));
    quitGameAction->setIcon(QIcon::fromTheme("application-exit"));
    connect(quitGameAction, SIGNAL(triggered( bool )), SLOT(close()));

    //TODO recent = KStandardGameAction::loadRecent(this, SLOT(openGame(const QUrl&)), actionCollection());
    //TODO recent->loadEntries(KGlobal::config()->group( QString() ));

    // Move
    undo = new QAction(tr("Undo"), this);
    undo->setShortcut(tr("Ctrl+Z"));
    undo->setIcon(QIcon::fromTheme("edit-undo"));
    connect(undo, SIGNAL(triggered( bool )), SLOT(undoMove()));

    redo = new QAction(tr("Redo"), this);
    redo->setShortcut(tr("Shift+Ctrl+Z"));
    redo->setIcon(QIcon::fromTheme("edit-redo"));
    connect(redo, SIGNAL(triggered( bool )), SLOT(redoMove()));

    chooseGameAction = new QAction(tr("&Choose Game..."), this);
    connect( chooseGameAction, SIGNAL(triggered(bool)), SLOT(chooseGame()) );

    changeGameTypeAction = new QAction(tr("Change Game Type..."), this);
    connect( changeGameTypeAction, SIGNAL(triggered(bool)), SLOT(slotShowGameSelectionScreen()) );

    randomSetAction = new QAction(tr("Random Cards"), this);
    connect( randomSetAction, SIGNAL(triggered(bool)), SLOT(slotPickRandom()) );
    randomSetAction->setShortcut( Qt::Key_F9 );

    selectDeckAction = new QAction(tr("Select Deck..."), this);
    connect( selectDeckAction, SIGNAL(triggered(bool)), SLOT(slotSelectDeck()) );
    selectDeckAction->setShortcut( Qt::Key_F10 );

    gameStatsAction = new QAction(tr("Statistics"), this);
    gameStatsAction->setIcon( QIcon::fromTheme("games-highscores") );
    connect( gameStatsAction, SIGNAL(triggered(bool)), SLOT(showStats()) );

    /*TODO gamehelpaction = actionCollection()->addAction("help_game");
    gamehelpaction->setIcon( KIcon("help-browser") );
    connect( gamehelpaction, SIGNAL(triggered(bool)), SLOT(helpGame()));
    gamehelpaction->setShortcuts( Qt::ControlModifier | Qt::ShiftModifier | Qt::Key_F1 );*/

    // Game type dependent actions
    hintaction = new QAction(tr("Hint"), this);
    hintaction->setShortcut(Qt::Key_H);
    hintaction->setIcon(QIcon::fromTheme("help-hint"));

    demoaction = new QAction(tr("Demo"), this);
    demoaction->setCheckable(true);
    demoaction->setIcon(QIcon::fromTheme("media-playback-start"));
    demoaction->setShortcut(Qt::Key_D);

    drawaction = new QAction(tr("Dra&w"), this);
    drawaction->setWhatsThis(tr("Take one or more cards from the deck, flip them, and place them in play"));
    // drawaction->setIcon( QIcon("kpat") );
    drawaction->setIcon(QIcon::fromTheme("go-next"));
    drawaction->setShortcut( Qt::Key_Space );

    dealaction = new QAction(tr("Dea&l"), this);
    dealaction->setWhatsThis(tr("Deal a new row of cards from the deck"));
    // dealaction->setIcon( KIcon("kpat") );
    dealaction->setIcon(QIcon::fromTheme("go-up"));
    dealaction->setShortcut( Qt::Key_Return );

    redealaction = new QAction(tr("&Redeal"), this);
    redealaction->setWhatsThis(tr("Collect the cards in play, shuffle them and redeal them"));
    redealaction->setIcon(QIcon::fromTheme("roll"));
    redealaction->setShortcut( Qt::Key_R );

    dropaction = new QAction(tr("Dro&p"), this);
    dropaction->setWhatsThis(tr("Automatically move cards to the foundation piles"));
    // dropaction->setIcon( KIcon("legalmoves") );
    dropaction->setIcon(QIcon::fromTheme("games-endturn"));
    dropaction->setShortcut( Qt::Key_P );

    // Configuration actions
    QSettings cg;
    cg.beginGroup(settings_group);

    autodropaction = new QAction(tr("&Enable Autodrop"), this);
    autodropaction->setCheckable(true);
    connect( autodropaction, SIGNAL(triggered(bool)), SLOT(enableAutoDrop(bool)) );
    autodropaction->setChecked( cg.value("Autodrop", true).toBool() );

    solveraction = new QAction(tr("E&nable Solver"), this);
    solveraction->setCheckable(true);
    connect( solveraction, SIGNAL(triggered(bool)), SLOT(enableSolver(bool)) );
    solveraction->setChecked( cg.value("Solver", true).toBool() );

    rememberstateaction = new QAction(tr("&Remember State on Exit"), this);
    rememberstateaction->setCheckable(true);
    connect( rememberstateaction, SIGNAL(triggered(bool)), SLOT(enableRememberState(bool)) );
    rememberstateaction->setChecked( cg.value("RememberStateOnExit", false).toBool() );

    foreach( const DealerInfo * di, DealerInfoList::self()->games() )
    {
        foreach( int id, di->ids )
            m_dealer_map.insert( id, di );
        if ( QString(di->name).toLower() == "freecell" )
            m_freeCellId = di->ids.first();
    }
    m_dealer_it = m_dealer_map.constEnd();

    m_cards = new cardMap();

    m_stack = new QStackedWidget;
    setCentralWidget( m_stack );

    QSize defaultSize = qApp->desktop()->availableGeometry().size() * 0.7;
    
    setupGUI();

    solverStatus = new QLabel(QString(), statusBar());
    solverStatus->setAlignment(Qt::AlignLeft | Qt::AlignVCenter);
    statusBar()->addWidget(solverStatus, 1);

    moveStatus = new QLabel(QString(), statusBar());
    moveStatus->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
    statusBar()->addWidget(moveStatus, 0);
    
    resize(800, 600);
}

void pWidget::setupGUI()
{
	qDebug() << "--------------------------" << Q_FUNC_INFO;
	gameMenu = new QMenu(tr("Game"), this);
	gameMenu->setSeparatorsCollapsible(true);
	
	gameMenu->addAction(newGameAction);
	gameMenu->addAction(loadGameAction);
	gameMenu->addAction(restartGameAction);
	gameMenu->addSeparator();
	gameMenu->addAction(saveGameAction);
	gameMenu->addSeparator();
	gameMenu->addAction(chooseGameAction);
	gameMenu->addAction(gameStatsAction);
	gameMenu->addSeparator();
	quitSeparator = gameMenu->addSeparator();
	gameMenu->addAction(quitGameAction);
	
	menuBar()->addMenu(gameMenu);
	
	QMenu* moveMenu = new QMenu(tr("Move"), this);
	moveMenu->addAction(undo);
	moveMenu->addAction(redo);
	
	menuBar()->addMenu(moveMenu);
	
	settingsMenu = new QMenu(tr("Settings"), this);
	settingsMenu->addAction(changeGameTypeAction);
	settingsMenu->addAction(autodropaction);
	settingsMenu->addAction(solveraction);
	settingsMenu->addAction(randomSetAction);
	settingsMenu->addAction(selectDeckAction);
	
	menuBar()->addMenu(settingsMenu);
	
	gameToolBar = addToolBar(tr("Show toolbar"));
	
	gameToolBar->addAction(newGameAction);
	gameToolBar->addAction(undo);
	gameToolBar->addSeparator();
	
	settingsMenu->addSeparator();
	settingsMenu->addAction(gameToolBar->toggleViewAction());
}

pWidget::~pWidget()
{
    //TODO recent->saveEntries(KGlobal::config()->group( QString() ));

    delete dill;
    delete m_cards;
}

void pWidget::undoMove() {
    if( m_dealer )
        m_dealer->undo();
}

void pWidget::redoMove() {
    if( m_dealer )
        m_dealer->redo();
}

void pWidget::helpGame()
{
    /*TODO if (m_dealer && m_dealer_map.contains(m_dealer->gameId()))
    {
        const DealerInfo * di = m_dealer_map.value(m_dealer->gameId());
        QString anchor = QString(di->name).toLower();
        anchor = anchor.remove('\'').replace('&', "and").replace(' ', '-');
        KToolInvocation::invokeHelp(anchor);
    }*/
}

void pWidget::enableAutoDrop(bool enable)
{
    QSettings cg;
    cg.beginGroup(settings_group);
    cg.setValue("Autodrop", enable);
    if (m_dealer)
        m_dealer->setAutoDropEnabled(enable);
    updateGameActionList();
}

void pWidget::enableSolver(bool enable)
{
    QSettings cg;
    cg.beginGroup(settings_group);
    cg.setValue("Solver", enable);
    solverStatus->setText(QString());
    if (m_dealer)
        m_dealer->setSolverEnabled(enable);
}

void pWidget::enableRememberState(bool enable)
{
    QSettings cg;
    cg.beginGroup(settings_group);
    cg.setValue( "RememberStateOnExit", enable );
}

void pWidget::newGame()
{
    if (m_dealer && m_dealer->allowedToStartNewGame())
        startRandom();
}

void pWidget::restart()
{
    startNew(-1);
}

void pWidget::startRandom()
{
    startNew(KRandom::random());
}

void pWidget::startNew(long gameNumber)
{
    solverStatus->setText(QString());
    m_dealer->startNew(gameNumber);
    setGameCaption();
}

void pWidget::slotPickRandom()
{
    QString theme = CardDeckInfo::randomFrontName();
    qDebug() << "theme" << theme;

    QSettings cs;
    cs.beginGroup(settings_group);
    CardDeckInfo::writeFrontTheme( cs, theme );

    cardMap::self()->updateTheme(cs);
    cardMap::self()->triggerRescale();
}

void pWidget::slotSelectDeck()
{
    QSettings cs;
    cs.beginGroup(settings_group);
    QPointer<KCardWidget> cardwidget = new KCardWidget();
    cardwidget->readSettings(cs);
    
    QString oldFrontName = cardwidget->frontName();
    QString oldBackName = cardwidget->backName();

    QPointer<KCardDialog> dlg = new KCardDialog(cardwidget);
    dlg->setParent(this, dlg->windowFlags());
    if (dlg->exec() == QDialog::Accepted && cardwidget)
    {
        // Always store the settings, as other things than only the deck may
        // have changed
        cardwidget->saveSettings(cs);
        cs.sync();

        if (cardwidget->frontName() != oldFrontName || cardwidget->backName() != oldBackName)
        {
            cardMap::self()->updateTheme(cs);
            cardMap::self()->triggerRescale();
        }
    }
    
    // cardWidget has been reparented to dlg, so deleting dlg cleans them both up.
    delete dlg;
}

void pWidget::setGameCaption()
{
    qDebug() << Q_FUNC_INFO;
    QString caption = tr("Qt Patience");
    if ( m_dealer )
    {
        const DealerInfo * di = m_dealer_map.value( m_dealer->gameId() );
        caption = QString("%1 - %2").arg(di->name).arg(m_dealer->gameNumber());
    }
    setWindowTitle( caption );
}

void pWidget::slotGameSelected(int id)
{
    if ( m_dealer_map.contains(id) )
    {
        newGameType(id);
        QTimer::singleShot(0 , this, SLOT( startRandom() ) );
    }
}

void pWidget::newGameType(int id)
{
    qDebug() << Q_FUNC_INFO;
    if ( !dill )
    {
        dill = new PatienceView(this, m_stack);
        m_stack->addWidget(dill);
    }

    // If we're replacing an existing DealerScene, record the stats of the
    // game already in progress.
    if ( m_dealer )
        m_dealer->recordGameStatistics();

    const DealerInfo * di = m_dealer_map.value(id, DealerInfoList::self()->games().first());
    m_dealer = di->createGame();
    m_dealer->setGameId( di->ids.first() );
    m_dealer->setAutoDropEnabled( autodropaction->isChecked() );
    m_dealer->setSolverEnabled( solveraction->isChecked() );

    dill->setScene( m_dealer );
    m_stack->setCurrentWidget(dill);

    //TODO gamehelpaction->setText(tr("Help &with %1").arg(tr(di->name).replace('&', "&&")));

    connect(m_dealer, SIGNAL(gameSolverStart()), SLOT(slotGameSolverStart()));
    connect(m_dealer, SIGNAL(gameSolverWon()), SLOT(slotGameSolverWon()));
    connect(m_dealer, SIGNAL(gameSolverLost()), SLOT(slotGameSolverLost()));
    connect(m_dealer, SIGNAL(gameSolverUnknown()), SLOT(slotGameSolverUnknown()));
    connect(m_dealer, SIGNAL(gameLost()), SLOT(slotGameLost()));
    connect(m_dealer, SIGNAL(updateMoves(int)), SLOT(slotUpdateMoves(int)));

    qDebug() << "setting labels text";

    solverStatus->setText(QString());
    solverStatus->setVisible(true);
    moveStatus->setText(QString());
    moveStatus->setVisible(true);

    updateActions();
}

void pWidget::slotShowGameSelectionScreen()
{
    if (!m_dealer || m_dealer->allowedToStartNewGame())
    {
        if (m_dealer)
        {
            m_dealer->recordGameStatistics();
            m_dealer = 0;
        }

        if (!m_bubbles)
        {
            m_bubbles = new DemoBubbles(m_stack);
            m_stack->addWidget(m_bubbles);
            connect( m_bubbles, SIGNAL(gameSelected(int)), SLOT(slotGameSelected(int)) );
        }
        m_stack->setCurrentWidget(m_bubbles);

        //TODO gamehelpaction->setText(tr("Help &with Current Game"));

        updateActions();

        setGameCaption();

        solverStatus->setVisible(false);
        moveStatus->setVisible(false);
    }
}

void pWidget::updateActions()
{
    // Enable/disable application actions that aren't appropriate on game
    // selection screen.
    newGameAction->setEnabled( m_dealer );
    restartGameAction->setEnabled( m_dealer );
    saveGameAction->setEnabled( m_dealer );
    chooseGameAction->setEnabled( m_dealer );
    changeGameTypeAction->setEnabled( m_dealer );
    //TODO gamehelpaction->setEnabled( m_dealer );

    // Initially disable game actions. They'll be reenabled through signals
    // if/when appropriate.
    undo->setEnabled( false );
    redo->setEnabled( false );
    hintaction->setEnabled( false );
    demoaction->setEnabled( false );
    drawaction->setEnabled( false );
    dealaction->setEnabled( false );
    redealaction->setEnabled( false );

    if ( m_dealer )
    {
        // If a dealer exists, connect the game actions to it.
        connect( m_dealer, SIGNAL(undoPossible(bool)), undo, SLOT(setEnabled(bool)) );
        connect( m_dealer, SIGNAL(redoPossible(bool)), redo, SLOT(setEnabled(bool)) );

        connect( hintaction, SIGNAL(triggered(bool)), m_dealer, SLOT(hint()) );
        connect( m_dealer, SIGNAL(hintPossible(bool)), hintaction, SLOT(setEnabled(bool)) );

        connect( demoaction, SIGNAL(triggered(bool)), m_dealer, SLOT(toggleDemo()) );
        connect( m_dealer, SIGNAL(demoActive(bool)), this, SLOT(toggleDemoAction(bool)) );
        connect( m_dealer, SIGNAL(demoPossible(bool)), demoaction, SLOT(setEnabled(bool)) );

        connect( dropaction, SIGNAL(triggered(bool)), m_dealer, SLOT(slotAutoDrop()) );

        if ( m_dealer->actions() & DealerScene::Draw )
        {
            connect( drawaction, SIGNAL(triggered(bool)), m_dealer, SLOT(newCards()) );
            connect( m_dealer, SIGNAL(newCardsPossible(bool)), drawaction, SLOT(setEnabled(bool)) );
        }
        else if ( m_dealer->actions() & DealerScene::Deal )
        {
            connect( dealaction, SIGNAL(triggered(bool)), m_dealer, SLOT(newCards()) );
            connect( m_dealer, SIGNAL(newCardsPossible(bool)), dealaction, SLOT(setEnabled(bool)) );
        }
        else if ( m_dealer->actions() & DealerScene::Redeal )
        {
            connect( redealaction, SIGNAL(triggered(bool)), m_dealer, SLOT(newCards()) );
            connect( m_dealer, SIGNAL(newCardsPossible(bool)), redealaction, SLOT(setEnabled(bool)) );
        }
    }
    /*TODO else
    {
        // Remove the game type specific options from the GUI.
        guiFactory()->unplugActionList( this, "dealer_options" );
        delete actionCollection()->action( "dealer_options" );
    }*/

    updateGameActionList();
}

void pWidget::updateGameActionList()
{
    for (int i = 0; i < gameActions.size(); i++)
    {
        gameMenu->removeAction(gameActions.at(i));
        gameToolBar->removeAction(gameActions.at(i));
    }

    dropaction->setEnabled( m_dealer && !m_dealer->autoDrop() );
    
    gameActions.clear();

    if ( m_dealer )
    {
        if ( m_dealer->actions() & DealerScene::Hint )
            gameActions.append( hintaction );
        if ( m_dealer->actions() & DealerScene::Demo )
            gameActions.append( demoaction );
        /*if ( m_dealer->actions() & DealerScene::Draw )
            gameActions.append( drawaction );
        if ( m_dealer->actions() & DealerScene::Deal )
            gameActions.append( dealaction );*/
        if ( m_dealer->actions() & DealerScene::Redeal )
            gameActions.append( redealaction );
        if ( !m_dealer->autoDrop() )
            gameActions.append( dropaction );

        gameMenu->insertActions(quitSeparator, gameActions);
        gameToolBar->addActions(gameActions);
    }
}

void pWidget::toggleDemoAction(bool active) 
{
    demoaction->setChecked( active );
    demoaction->setIcon(QIcon::fromTheme(active ? "media-playback-pause" : "media-playback-start"));
}

void pWidget::saveNewToolbarConfig()
{
    //TODO QMainWindow::saveNewToolbarConfig();
    updateGameActionList();
}

void pWidget::closeEvent(QCloseEvent *e)
{
    QFile savedState(QDir::homePath() + "/.qpat/" + saved_state_file); //TODO KStandardDirs::locateLocal("appdata", saved_state_file));
    if (savedState.exists())
        savedState.remove();

    if ( m_dealer )
    {
        if (rememberstateaction->isChecked())
        {
            m_dealer->save_it(savedState.fileName());
        }
        else
        {
            // If there's a game in progress and we aren't going to save it
            // then record its statistics, since the DealerScene will be destroyed
            // shortly.
            m_dealer->recordGameStatistics();
        }
    }

    QMainWindow::closeEvent(e);
}

void pWidget::chooseGame()
{
    if (m_dealer)
    {
        QString text = (m_dealer->gameId() == m_freeCellId)
                       ? tr("Enter a game number (Freecell deals are the same as in the Freecell FAQ):")
                       : tr("Enter a game number:");
        bool ok;
        long number = QInputDialog::getText(this, tr("Game Number"),
                                            text,
                                            QLineEdit::Normal,
                                            QString::number(m_dealer->gameNumber())).toLong(&ok);

        if (ok && m_dealer->allowedToStartNewGame())
            startNew(number);
    }
}

bool pWidget::openGame(const QString& filePath, bool addToRecentFiles)
{
	QString error;
	
	QFile of(filePath);
	of.open(QIODevice::ReadOnly);
	QDomDocument doc;
	
	if (doc.setContent(&of, &error))
	{
		if (doc.documentElement().tagName() == "dealer")
		{
		int id = doc.documentElement().attribute("id").toInt();
		if (m_dealer_map.contains(id))
		{
			// Only ask for permission after we've determined the save game
			// file is good.
			if (!m_dealer || m_dealer->allowedToStartNewGame())
			{
			// If the file has a game number, then load it.
			// If it only contains an ID, just launch a new game with that ID.
			if (doc.documentElement().hasAttribute("number"))
			{
				newGameType(id);
				// for old spider and klondike
				m_dealer->mapOldId(id);
				m_dealer->openGame(doc);
				setGameCaption();
			}
			else
			{
				slotGameSelected(id);
			}
	
			/*TODO if ( addToRecentFiles )
				recent->addUrl(url);*/
			}
		}
		else
		{
			error = tr("The saved game is of an unknown type.");
		}
		}
		else
		{
		error = tr("The file is not a QPatience saved game.");
		}
	}
	else
	{
		error = tr("The following error occurred while reading the file:\n\"%1\"").arg(error);
	}


    if (!error.isEmpty())
        QMessageBox::critical(this, tr("Error"), error);

    return error.isEmpty();
}

void pWidget::openGame()
{
    QString filePath = QFileDialog::getOpenFileName(this, tr("Select game file"));
    if (!filePath.isEmpty())
        openGame(filePath);
}

void pWidget::saveGame()
{
    if (m_dealer)
    {
        QString filePath = QFileDialog::getSaveFileName(this, tr("Select game file"));
        QString tmpFile = m_dealer->save_it(filePath);
        //TODO recent->addUrl(url);
    }
}

void pWidget::showStats()
{
    GameStatsImpl dlg(this);
    if (m_dealer)
        dlg.showGameType(m_dealer->gameId());
    dlg.exec();
}

void pWidget::slotGameSolverStart()
{
    solverStatus->setText(tr("Solver: Calculating..."));
}

void pWidget::slotGameSolverWon()
{
    solverStatus->setText(tr("Solver: This game is winnable."));
}

void pWidget::slotGameSolverLost()
{
    solverStatus->setText(tr("Solver: This game is not winnable in its current state."));
}

void pWidget::slotGameSolverUnknown()
{
    solverStatus->setText(tr("Solver: Unable to determine if this game is winnable."));
}

void pWidget::slotGameLost()
{
    solverStatus->setText(tr("Solver: This game is lost."));
}

void pWidget::slotUpdateMoves(int moves)
{
    moveStatus->setText(tr("%n move(s)", "", moves));
}

void pWidget::slotSnapshot()
{
    if ( m_dealer_it == m_dealer_map.constEnd() )
    {
        // first call
        m_dealer_it = m_dealer_map.constBegin();
    }

    newGameType(m_dealer_it.key());
    startRandom();
    m_dealer_it++;
    QTimer::singleShot( 200, this, SLOT( slotSnapshot2() ) );
}

void pWidget::slotSnapshot2()
{
    qDebug() << Q_FUNC_INFO;
    if ( m_dealer->waiting() )
    {
            QTimer::singleShot( 100, this, SLOT( slotSnapshot2() ) );
            return;
    }
    QImage img = QImage( dill->size(), QImage::Format_ARGB32 );
    img.fill( qRgba( 0, 0, 255, 0 ) );
    m_dealer->createDump( &img );
    img = img.scaled( 320, 320, Qt::KeepAspectRatio, Qt::SmoothTransformation );
    img.save( QString( "demo_%1.png" ).arg( m_dealer->gameId() ) );
    if ( m_dealer_it != m_dealer_map.constEnd() )
        QTimer::singleShot( 200, this, SLOT( slotSnapshot() ) );
}

void pWidget::addGameOptionsAction(QAction * a)
{
	settingsMenu->insertAction(autodropaction, a);
}
